



pip check
IF %ERRORLEVEL% NEQ 0 exit /B 1
python -m pytest lime/tests -vv -k "not (test_indexed_string_callable or test_indexed_string_regex or test_lime_explainer_entropy_discretizer)"
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
